"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
const utils_1 = require("./utils");
const camel_case_1 = require("camel-case");
class ErudaWebpackPlugin {
    constructor(options = {}) {
        const isProdEnv = process.env.NODE_ENV === 'production';
        const { enable = !isProdEnv, plugins = [], container = null, tool = null, autoScale = true, useShadowDom = true, } = options;
        let { exclude = [] } = options;
        if (utils_1.isString(exclude)) {
            exclude = [exclude];
        }
        else if (utils_1.isRegExp(exclude)) {
            exclude = [exclude];
        }
        else if (!Array.isArray(exclude)) {
            utils_1.log.warn(`\n[vue-cli-plugin-eruda] Invalid options: "exclude" must be regexp/regexp[] \n`);
            exclude = [];
        }
        this.options = {
            // plugin options
            enable,
            plugins,
            exclude,
            // eruda init options
            container,
            tool,
            autoScale,
            useShadowDom,
        };
        // enable by default when NODE_ENV !== 'production'
        if (this.options.enable) {
            this._writeErudaEntry();
        }
    }
    // https://github.com/webpack/webpack/issues/6516
    _amendEntry(entry) {
        // vue-cli does not allow function type
        if (utils_1.isEntryFunc(entry))
            return (...args) => Promise.resolve(entry(...args)).then(this._amendEntry.bind(this));
        // webpack object entry type: { <key> string | [string] }
        if (utils_1.isEntry(entry)) {
            const newEntryObj = {};
            Object.keys(entry).forEach((key) => {
                newEntryObj[key] = this._injectErudaEntryPath(entry[key]);
            });
            return newEntryObj;
        }
        if (entry) {
            return this._injectErudaEntryPath(entry);
        }
        return entry;
    }
    _injectErudaEntryPath(entry) {
        if (utils_1.isString(entry)) {
            return this._inExclude([entry]) ? [entry] : [this.erudaEntryPath, entry];
        }
        else if (Array.isArray(entry)) {
            return this._inExclude(entry) ? entry : [this.erudaEntryPath, ...entry];
        }
        return entry;
    }
    _inExclude(entry) {
        const { exclude = [] } = this.options;
        for (let i = 0; i < exclude.length; i++) {
            const rule = exclude[i];
            for (let j = 0; j < entry.length; j++) {
                const path = entry[j];
                if ((utils_1.isString(rule) && path.startsWith(rule)) ||
                    (utils_1.isRegExp(rule) && rule.test(path))) {
                    return true;
                }
            }
        }
        return false;
    }
    _writeErudaEntry() {
        const pluginStr = this._getPlugin();
        const optionsStr = this._getInitOptions();
        const erudaStr = `var eruda = require("eruda");window.eruda === undefined && (window.eruda = eruda);`;
        const initStr = `eruda.init(${optionsStr});`;
        fs.writeFileSync(path.resolve(__dirname, './eruda-entry.js'), erudaStr + initStr + pluginStr, {
            encoding: 'utf8',
            flag: 'w',
        });
        this.erudaEntryPath = require.resolve('./eruda-entry.js');
    }
    _getInitOptions() {
        const { container, tool, autoScale, useShadowDom } = this.options;
        const option = {
            container,
            autoScale,
            useShadowDom,
        };
        if (tool)
            option.tool = tool;
        return JSON.stringify(option);
    }
    _getPlugin() {
        const plugins = this.options.plugins;
        let pluginsStr = '';
        if (Array.isArray(plugins)) {
            [...plugins].forEach((val) => {
                let pluginName = val;
                let hasModule = false;
                if (!utils_1.isString(val))
                    return;
                if (!val.startsWith('eruda'))
                    pluginName = `eruda-${val}`;
                try {
                    require.resolve(pluginName);
                    hasModule = true;
                }
                catch (err) {
                    utils_1.log.warn(`\n[vue-cli-plugin-eruda] Error: Cannot find eruda plugin "${val}". \nYou may need to install it.\n`);
                }
                if (!hasModule)
                    return;
                const pluginCamelCase = camel_case_1.camelCase(pluginName);
                pluginsStr += `var ${pluginCamelCase} = require("${pluginName}");eruda.add(${pluginCamelCase});`;
            });
        }
        return pluginsStr;
    }
    apply(compiler) {
        if (this.options.enable) {
            compiler.options.entry = this._amendEntry(compiler.options.entry);
        }
    }
}
exports.default = ErudaWebpackPlugin;
